Package: netboost
Type: Package
Title: Network Analysis Supported by Boosting
Version: 2.19.0
Date: 2024-06-12
Authors@R: c(person("Pascal", "Schlosser", role = c("aut", "cre"),
                    email = "pascal.schlosser@uniklinik-freiburg.de"),
             person("Jochen", "Knaus", role = c("aut", "ctb"),
                    email = "jo@imbi.uni-freiburg.de"),
             person("Yaniv", "Loewenstein", role = c("aut")))
Description: Boosting supported network analysis for high-dimensional omics applications. 
    This package comes bundled with the MC-UPGMA clustering package by Yaniv Loewenstein.
Depends: R (>= 4.0.0)
biocViews: Software, StatisticalMethod, GraphAndNetwork, Network,
        Clustering, DimensionReduction, BiomedicalInformatics,
        Epigenetics, Metabolomics, Transcriptomics
Imports: Rcpp, RcppParallel, parallel, grDevices, graphics, stats,
        utils, dynamicTreeCut, WGCNA, impute, colorspace, methods,
        BiocStyle, R.utils
LinkingTo: Rcpp, RcppParallel
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
SystemRequirements: GNU make, Bash, Perl, Gzip
OS_type: unix
ByteCompile: yes
NeedsCompilation: yes
LazyData: true
License: GPL-3
URL: https://bioconductor.org/packages/release/bioc/html/netboost.html
BugReports: pascal.schlosser@uniklinik-freiburg.de
Encoding: UTF-8
RoxygenNote: 7.2.3
git_url: https://git.bioconductor.org/packages/netboost
git_branch: devel
git_last_commit: cd295da
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-30
Packaged: 2025-10-30 22:11:46 UTC; biocbuild
Author: Pascal Schlosser [aut, cre],
  Jochen Knaus [aut, ctb],
  Yaniv Loewenstein [aut]
Maintainer: Pascal Schlosser <pascal.schlosser@uniklinik-freiburg.de>
