#' @name tcga_aml_meth_rna_chr18
#' @title TCGA RNA and methylation measurement on a subset of
#' chromosome 18 for 80 AML patients.
#' @docType data
#' @usage tcga_aml_meth_rna_chr18
#' @format A data frame with 80 rows (patients) and 500 variables (features).
#' @source \url{https://portal.gdc.cancer.gov/}
NULL
