% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{cpp_filter_end}
\alias{cpp_filter_end}
\title{Boosting cleanup (required to free memory)}
\usage{
cpp_filter_end()
}
\value{
none
}
\description{
Boosting cleanup (required to free memory)
}
